import json
from datetime import datetime
from typing import Dict, Any, Optional, List

from urllib3 import PoolManager

from kakaowork.consts import (
    LIMIT,
    BASE_URL,
    BASE_PATH_USERS,
    BASE_PATH_CONVERSATIONS,
    BASE_PATH_MESSAGES,
    BASE_PATH_DEPARTMENTS,
    BASE_PATH_SPACES,
    BASE_PATH_BOTS,
)
from kakaowork.models import (
    BaseResponse,
    UserResponse,
    UserListResponse,
    ConversationResponse,
    ConversationListResponse,
    MessageResponse,
    DepartmentListResponse,
    SpaceResponse,
    BotResponse,
)
from kakaowork.blockkit import Block


class Kakaowork:
    class Users:
        def __init__(self, client: 'Kakaowork', *, base_path: Optional[str] = BASE_PATH_USERS):
            self.client = client
            self.base_path = base_path

        def info(self, user_id: int) -> UserResponse:
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.info',
                fields={'user_id': user_id},
            )
            return UserResponse.from_json(r.data)

        def find_by_email(self, email: str) -> UserResponse:
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.find_by_email',
                fields={'email': email},
            )
            return UserResponse.from_json(r.data)

        def find_by_phone_number(self, phone_number: str) -> UserResponse:
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.find_by_phone_number',
                fields={'phone_number': phone_number},
            )
            return UserResponse.from_json(r.data)

        def list(self, *, cursor: Optional[str] = None, limit: Optional[int] = LIMIT) -> UserListResponse:
            fields: Dict[str, Any] = {'cursor': cursor} if cursor else {'limit': str(limit)}
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.list',
                fields=fields,
            )
            return UserListResponse.from_json(r.data)

        def set_work_time(self, *, user_id: int, work_start_time: datetime, work_end_time: datetime) -> BaseResponse:
            payload = {
                'user_id': user_id,
                'work_start_time': int(work_start_time.timestamp()),
                'work_end_time': int(work_end_time.timestamp()),
            }
            r = self.client.http.request(
                'POST',
                f'{self.client.base_url}{self.base_path}.set_work_time',
                body=json.dumps(payload).encode('utf-8'),
            )
            return BaseResponse.from_json(r.data)

        def set_vacation_time(self, *, user_id: int, vacation_start_time: datetime, vacation_end_time: datetime) -> BaseResponse:
            payload = {
                'user_id': user_id,
                'vacation_start_time': int(vacation_start_time.timestamp()),
                'vacation_end_time': int(vacation_end_time.timestamp()),
            }
            r = self.client.http.request(
                'POST',
                f'{self.client.base_url}{self.base_path}.set_vacation_time',
                body=json.dumps(payload).encode('utf-8'),
            )
            return BaseResponse.from_json(r.data)

    class Conversations:
        def __init__(self, client: 'Kakaowork', *, base_path: Optional[str] = BASE_PATH_CONVERSATIONS):
            self.client = client
            self.base_path = base_path

        def open(self, *, user_ids: List[int]) -> ConversationResponse:
            payload: Dict[str, Any] = {'user_id': user_ids[0]} if len(user_ids) == 1 else {'user_ids': user_ids}
            r = self.client.http.request(
                'POST',
                f'{self.client.base_url}{self.base_path}.open',
                body=json.dumps(payload).encode('utf-8'),
            )
            return ConversationResponse.from_json(r.data)

        def list(self, *, cursor: Optional[str] = None, limit: Optional[int] = LIMIT) -> ConversationListResponse:
            fields = {'cursor': cursor} if cursor else {'limit': str(limit)}
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.list',
                fields=fields,
            )
            return ConversationListResponse.from_json(r.data)

        def users(self, *, conversation_id: int) -> UserListResponse:
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}/{conversation_id}/users',
            )
            return UserListResponse.from_json(r.data)

        def invite(self, *, conversation_id: int, user_ids: List[int]) -> BaseResponse:
            payload = {'user_ids': user_ids}
            r = self.client.http.request(
                'POST',
                f'{self.client.base_url}{self.base_path}/{conversation_id}/invite',
                body=json.dumps(payload).encode('utf-8'),
            )
            return BaseResponse.from_json(r.data)

        def kick(self, *, conversation_id: int, user_ids: List[int]) -> BaseResponse:
            payload = {'user_ids': user_ids}
            r = self.client.http.request(
                'POST',
                f'{self.client.base_url}{self.base_path}/{conversation_id}/kick',
                body=json.dumps(payload).encode('utf-8'),
            )
            return BaseResponse.from_json(r.data)

    class Messages:
        def __init__(self, client: 'Kakaowork', *, base_path: Optional[str] = BASE_PATH_MESSAGES):
            self.client = client
            self.base_path = base_path

        def send(self, *, conversation_id: int, text: str, blocks: Optional[List[Block]] = None) -> MessageResponse:
            payload = {
                'conversation_id': conversation_id,
                'text': text,
                'blocks': blocks or [],
            }
            r = self.client.http.request(
                'POST',
                f'{self.client.base_url}{self.base_path}.send',
                body=json.dumps(payload).encode('utf-8'),
            )
            return MessageResponse.from_json(r.data)

    class Departments:
        def __init__(self, client: 'Kakaowork', *, base_path: Optional[str] = BASE_PATH_DEPARTMENTS):
            self.client = client
            self.base_path = base_path

        def list(self, *, cursor: Optional[str] = None, limit: Optional[int] = LIMIT) -> DepartmentListResponse:
            fields = {'cursor': cursor} if cursor else {'limit': str(limit)}
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.list',
                fields=fields,
            )
            return DepartmentListResponse.from_json(r.data)

    class Spaces:
        def __init__(self, client: 'Kakaowork', *, base_path: Optional[str] = BASE_PATH_SPACES):
            self.client = client
            self.base_path = base_path

        def info(self) -> SpaceResponse:
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.info',
            )
            return SpaceResponse.from_json(r.data)

    class Bots:
        def __init__(self, client: 'Kakaowork', *, base_path: Optional[str] = BASE_PATH_BOTS):
            self.client = client
            self.base_path = base_path

        def info(self) -> BotResponse:
            r = self.client.http.request(
                'GET',
                f'{self.client.base_url}{self.base_path}.info',
            )
            return BotResponse.from_json(r.data)

    def __init__(self, *, app_key: str, base_url: Optional[str] = BASE_URL):
        self.app_key = app_key
        self.base_url = base_url
        self.http = PoolManager(headers=self.headers, retries=3, maxsize=5)

    @property
    def headers(self) -> Dict[str, Any]:
        return {
            'Authorization': f'Bearer {self.app_key}',
            'Content-Type': 'application/json; charset=utf-8',
        }

    @property
    def users(self) -> Users:
        return self.Users(self)

    @property
    def conversations(self) -> Conversations:
        return self.Conversations(self)

    @property
    def messages(self) -> Messages:
        return self.Messages(self)

    @property
    def departments(self) -> Departments:
        return self.Departments(self)

    @property
    def spaces(self) -> Spaces:
        return self.Spaces(self)

    @property
    def bots(self) -> Bots:
        return self.Bots(self)



import json
from datetime import datetime
from typing import Union, Any, Dict

from pytz import utc

from kakaowork.consts import KST


def text2dict(text: Union[str, bytes]) -> Dict[str, Any]:
    if not text:
        return {}
    json_str = text.decode('utf-8') if isinstance(text, bytes) else text
    return json.loads(json_str)


def exist_kv(key: str, node: Dict[str, Any]) -> bool:
    if key in node and node[key]:
        return True
    return False


def to_kst(timestamp: Union[int, datetime]) -> datetime:
    if isinstance(timestamp, int):
        return datetime.fromtimestamp(timestamp, tz=utc).astimezone(KST)
    elif isinstance(timestamp, datetime):
        # If the timestamp is naive then just replace tzinfo to KST
        # ref: https://docs.python.org/3/library/datetime.html#determining-if-an-object-is-aware-or-naive
        if timestamp.tzinfo is None or timestamp.tzinfo.utcoffset(timestamp) is None:
            return KST.localize(timestamp)
        else:
            return timestamp.astimezone(KST)
    raise ValueError('Unsupported timestamp type')



import os
import json
from enum import unique
from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional, NamedTuple, Type, Union
from urllib.parse import urlparse

from kakaowork.consts import StrEnum
from kakaowork.exceptions import InvalidBlock, InvalidBlockType


@unique
class BlockType(StrEnum):
    TEXT = "text"
    IMAGE_LINK = "image_link"
    BUTTON = "button"
    DIVIDER = "divider"
    HEADER = "header"
    ACTION = "action"
    DESCRIPTION = "description"
    SECTION = "section"
    CONTEXT = "context"
    LABEL = "label"
    INPUT = "input"
    SELECT = "select"

    @classmethod
    def block_cls(cls, block_type: Union[str, 'BlockType']) -> Type['Block']:
        bt = cls(block_type) if isinstance(block_type, str) else block_type
        if bt == cls.TEXT:
            return TextBlock
        elif bt == cls.IMAGE_LINK:
            return ImageLinkBlock
        elif bt == cls.BUTTON:
            return ButtonBlock
        elif bt == cls.DIVIDER:
            return DividerBlock
        elif bt == cls.HEADER:
            return HeaderBlock
        elif bt == cls.ACTION:
            return ActionBlock
        elif bt == cls.DESCRIPTION:
            return DescriptionBlock
        elif bt == cls.SECTION:
            return SectionBlock
        elif bt == cls.CONTEXT:
            return ContextBlock
        elif bt == cls.LABEL:
            return LabelBlock
        elif bt == cls.INPUT:
            return InputBlock
        elif bt == cls.SELECT:
            return SelectBlock
        raise InvalidBlockType()


@unique
class ButtonStyle(StrEnum):
    DEFAULT = "default"
    PRIMARY = "primary"
    DANGER = "danger"


@unique
class ButtonActionType(StrEnum):
    OPEN_INAPP_BROWSER = "open_inapp_browser"
    OPEN_SYSTEM_BROWSER = "open_system_browser"
    OPEN_EXTERNAL_APP = "open_external_app"
    SUBMIT_ACTION = "submit_action"
    CALL_MODAL = "call_modal"


@unique
class HeaderStyle(StrEnum):
    BLUE = 'blue'
    RED = 'red'
    YELLOW = 'yellow'


@unique
class BlockKitType(StrEnum):
    MESSAGE = 'message'
    MODAL = 'modal'


class SelectBlockOption(NamedTuple):
    text: str
    value: str


def json_default(value: Any) -> Any:
    if isinstance(value, Block):
        return value.to_dict()
    raise TypeError('not JSON serializable')


class Block(ABC):
    def __init__(self, *, block_type: BlockType):
        self.block_type = block_type
        self.block_vars: Dict[str, Any] = {}

    def __str__(self):
        return self.to_json()

    def __repr__(self):
        return str(self)

    def __eq__(self, value: object) -> bool:
        if not isinstance(value, Block):
            return False
        if self.block_type != value.block_type:
            return False
        if self.block_vars != value.block_vars:
            return False
        return True

    def to_dict(self) -> Dict[str, Any]:
        block_vars = {key: value for key, value in self.block_vars.items() if value is not None}
        return dict(type=self.block_type, **block_vars)

    def to_json(self) -> str:
        return json.dumps(self.to_dict(), default=json_default)

    @abstractmethod
    def validate(self) -> bool:
        raise NotImplementedError()


class TextBlock(Block):
    max_len_text = 500

    def __init__(self, *, text: str, markdown: Optional[bool] = False):
        super().__init__(block_type=BlockType.TEXT)
        self.block_vars = {
            'text': text,
            'markdown': markdown or False,
        }

    @property
    def text(self) -> str:
        return self.block_vars['text']

    @property
    def markdown(self) -> bool:
        return self.block_vars['markdown']

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        return True


class ImageLinkBlock(Block):
    def __init__(self, *, url: str):
        super().__init__(block_type=BlockType.IMAGE_LINK)
        self.block_vars = {
            'url': url,
        }

    @property
    def url(self) -> str:
        return self.block_vars['url']

    def validate(self) -> bool:
        if not self.url:
            return False
        o = urlparse(self.url)
        if not (o.scheme and o.netloc and o.path):
            return False
        if not os.path.splitext(o.path)[1]:
            return False
        return True


class ButtonBlock(Block):
    max_len_text = 20

    def __init__(self,
                 *,
                 text: str,
                 style: ButtonStyle,
                 action_type: Optional[ButtonActionType] = None,
                 action_name: Optional[str] = None,
                 value: Optional[str] = None):
        super().__init__(block_type=BlockType.BUTTON)
        self.block_vars = {
            'text': text,
            'style': style,
            'action_type': action_type,
            'action_name': action_name,
            'value': value,
        }

    @property
    def text(self) -> str:
        return self.block_vars['text']

    @property
    def style(self) -> ButtonStyle:
        return self.block_vars['style']

    @property
    def action_type(self) -> ButtonActionType:
        return self.block_vars['action_type']

    @property
    def action_name(self) -> str:
        return self.block_vars['action_name']

    @property
    def value(self) -> str:
        return self.block_vars['value']

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        if not self.style:
            return False
        return True


class DividerBlock(Block):
    def __init__(self) -> None:
        super().__init__(block_type=BlockType.DIVIDER)

    def validate(self) -> bool:
        return True


class HeaderBlock(Block):
    max_len_text = 20

    def __init__(self, *, text: str, style: Optional[HeaderStyle] = None):
        super().__init__(block_type=BlockType.HEADER)
        self.block_vars = {
            'text': text,
            'style': style,
        }

    @property
    def text(self) -> str:
        return self.block_vars['text']

    @property
    def style(self) -> HeaderStyle:
        return self.block_vars['style']

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        if not self.style:
            return False
        return True


class ActionBlock(Block):
    max_len_elements = 3

    def __init__(self, *, elements: List[ButtonBlock]):
        super().__init__(block_type=BlockType.ACTION)
        self.block_vars = {
            'elements': elements,
        }

    @property
    def elements(self) -> List[ButtonBlock]:
        return self.block_vars['elements']

    def validate(self) -> bool:
        if not self.elements or len(self.elements) > self.max_len_elements:
            return False
        return True


class DescriptionBlock(Block):
    max_len_term = 10

    def __init__(self, *, term: str, content: TextBlock, accent: Optional[bool] = False):
        super().__init__(block_type=BlockType.DESCRIPTION)
        self.block_vars = {
            'term': term,
            'content': content,
            'accent': accent or False,
        }

    @property
    def term(self) -> str:
        return self.block_vars['term']

    @property
    def content(self) -> TextBlock:
        return self.block_vars['content']

    @property
    def accent(self) -> bool:
        return self.block_vars['accent']

    def validate(self) -> bool:
        if not self.term or len(self.term) > self.max_len_term:
            return False
        if not self.content:
            return False
        if self.accent is None:
            return False
        return True


class SectionBlock(Block):
    def __init__(self, *, content: TextBlock, accessory: ImageLinkBlock):
        super().__init__(block_type=BlockType.SECTION)
        self.block_vars = {
            'content': content,
            'accessory': accessory,
        }

    @property
    def content(self) -> TextBlock:
        return self.block_vars['content']

    @property
    def accessory(self) -> ImageLinkBlock:
        return self.block_vars['accessory']

    def validate(self) -> bool:
        if not self.content:
            return False
        if not self.accessory:
            return False
        return True


class ContextBlock(Block):
    def __init__(self, *, content: TextBlock, image: ImageLinkBlock):
        super().__init__(block_type=BlockType.CONTEXT)
        self.block_vars = {
            'content': content,
            'image': image,
        }

    @property
    def content(self) -> TextBlock:
        return self.block_vars['content']

    @property
    def image(self) -> ImageLinkBlock:
        return self.block_vars['image']

    def validate(self) -> bool:
        if not self.content:
            return False
        if not self.image:
            return False
        return True


class LabelBlock(Block):
    max_len_text = 200

    def __init__(self, *, text: str, markdown: bool):
        super().__init__(block_type=BlockType.LABEL)
        self.block_vars = {
            'text': text,
            'markdown': markdown,
        }

    @property
    def text(self) -> str:
        return self.block_vars['text']

    @property
    def markdown(self) -> bool:
        return self.block_vars['markdown']

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        if self.markdown is None:
            return False
        return True


class InputBlock(Block):
    max_len_placeholder = 50

    def __init__(self, *, name: str, required: Optional[bool] = False, placeholder: Optional[str] = None):
        super().__init__(block_type=BlockType.INPUT)
        self.block_vars = {
            'name': name,
            'required': required or False,
            'placeholder': placeholder,
        }

    @property
    def name(self) -> str:
        return self.block_vars['name']

    @property
    def required(self) -> Optional[bool]:
        return self.block_vars['required']

    @property
    def placeholder(self) -> Optional[str]:
        return self.block_vars['placeholder']

    def validate(self) -> bool:
        if not self.name:
            return False
        if self.placeholder and len(self.placeholder) > self.max_len_placeholder:
            return False
        return True


class SelectBlock(Block):
    max_len_options = 30
    max_len_placeholder = 50

    def __init__(self, *, name: str, options: List[SelectBlockOption], required: Optional[bool] = False, placeholder: Optional[str] = None):
        super().__init__(block_type=BlockType.SELECT)
        self.block_vars = {
            'name': name,
            'options': options,
            'required': required or False,
            'placeholder': placeholder,
        }

    @property
    def name(self) -> str:
        return self.block_vars['name']

    @property
    def options(self) -> List[SelectBlockOption]:
        return self.block_vars['options']

    @property
    def required(self) -> Optional[bool]:
        return self.block_vars['required']

    @property
    def placeholder(self) -> Optional[str]:
        return self.block_vars['placeholder']

    def validate(self) -> bool:
        if not self.name:
            return False
        if not self.options or len(self.options) > self.max_len_options:
            return False
        if self.placeholder and len(self.placeholder) > self.max_len_placeholder:
            return False
        return True


class BlockKitBuilder:
    def __init__(self, *, kit_type: BlockKitType):
        self.kit_type = kit_type
        self.reset()

    def reset(self):
        self.kit_vars: Dict[str, Any] = {
            'blocks': [],
        }

    @property
    def text(self) -> str:
        if self.kit_type != BlockKitType.MESSAGE:
            raise InvalidBlockType("It can be set only for message type")
        return self.kit_vars['text'] if 'text' in self.kit_vars else ''

    @text.setter
    def text(self, value: str):
        if self.kit_type != BlockKitType.MESSAGE:
            raise InvalidBlockType("It can be set only for message type")
        self.kit_vars['text'] = value

    @property
    def title(self) -> str:
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.kit_vars['title'] if 'title' in self.kit_vars else ''

    @title.setter
    def title(self, value: str):
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.kit_vars['title'] = value

    @property
    def accept(self) -> str:
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.kit_vars['accept'] if 'accept' in self.kit_vars else ''

    @accept.setter
    def accept(self, value: str):
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.kit_vars['accept'] = value

    @property
    def decline(self) -> str:
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.kit_vars['decline'] if 'decline' in self.kit_vars else ''

    @decline.setter
    def decline(self, value: str):
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.kit_vars['decline'] = value

    @property
    def value(self) -> str:
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.kit_vars['value'] if 'value' in self.kit_vars else ''

    @value.setter
    def value(self, value: str):
        if self.kit_type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.kit_vars['value'] = value

    @property
    def blocks(self) -> List[Block]:
        return self.kit_vars['blocks'] if 'blocks' in self.kit_vars and self.kit_vars['blocks'] else []

    @blocks.setter
    def blocks(self, value: List[Block]):
        self.kit_vars['blocks'] = []
        for item in value:
            self.add_block(item)

    def add_block(self, block: Block):
        if not block.validate():
            raise InvalidBlock()
        self.kit_vars['blocks'].append(block)

    def to_dict(self) -> Dict[str, Any]:
        return {key: value for key, value in self.kit_vars.items() if value is not None}

    def to_json(self) -> str:
        return json.dumps(self.to_dict(), default=json_default)



from kakaowork.blockkit import (BlockType, ButtonStyle, ButtonActionType, HeaderStyle, BlockKitType, SelectBlockOption, TextBlock, ImageLinkBlock, ButtonBlock,
                                DividerBlock, HeaderBlock, ActionBlock, DescriptionBlock, SectionBlock, ContextBlock, LabelBlock, InputBlock, SelectBlock,
                                BlockKitBuilder)

from kakaowork.exceptions import (KakaoworkError, InvalidBlock, InvalidBlockType, NoValueError)

from kakaowork.client import Kakaowork

from kakaowork.models import (ErrorCode, ConversationType, ColorTone, ProfileNameFormat, ProfilePositionFormat, BotStatus, ErrorField, UserIdentificationField,
                              UserField, ConversationField, MessageField, DepartmentField, SpaceField, BotField, BaseResponse, UserResponse, UserListResponse,
                              ConversationResponse, ConversationListResponse, MessageResponse, DepartmentListResponse, SpaceResponse, BotResponse)

__version__ = '0.1.3'



import sys


def main():
    try:
        from kakaowork.command import cli, CLIOptions
        cli(obj=CLIOptions())
    except ImportError:
        sys.exit('Does not support CLI')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kakaowork']

package_data = \
{'': ['*']}

install_requires = \
['pytz', 'urllib3>=1.26.4,<2.0.0']

extras_require = \
{'cli': ['click>=7.1.2,<8.0.0']}

entry_points = \
{'console_scripts': ['kakaowork = kakaowork.__main__:main']}

setup_kwargs = {
    'name': 'kakaowork',
    'version': '0.1.3',
    'description': 'Kakaowork Python client',
    'long_description': '# kakaowork-py\n\n(Unofficial) Kakaowork Python client\n\n[![CI](https://github.com/skyoo2003/kakaowork-py/actions/workflows/ci.yml/badge.svg?branch=master)](https://github.com/skyoo2003/kakaowork-py/actions/workflows/ci.yml) [![Documentation Status](https://readthedocs.org/projects/kakaowork-py/badge/?version=latest)](https://kakaowork-py.readthedocs.io/en/latest)\n [![codecov](https://codecov.io/gh/skyoo2003/kakaowork-py/branch/master/graph/badge.svg?token=J6NQHDJEMZ)](https://codecov.io/gh/skyoo2003/kakaowork-py)\n\n__Table of Contents__\n\n- [Prerequisites](#prerequisites)\n- [Installation](#installation)\n- [Usage](#usage)\n- [Contributing](#contributing)\n- [License](#license)\n\n## Prerequisites\n\n- Python >= 3.6.1\n\n## Installation\n\nUse the package manager [pip](https://pip.pypa.io/en/stable/) to install kakaowork-py\n\n```bash\npip install kakaowork\n```\n\nIf you want to use CLI, install with the extras \'cli\'\n\n```bash\npip install kakaowork[cli]\n```\n\n## Usage\n\n```python\nfrom kakaowork import Kakaowork\n\nclient = Kakaowork(app_key="your_app_key")\nr = client.users.list(limit=10) # get a response of users using limit\nprint(r.users)\nwhile r.cursor: # loop until it does not to exist\n  print(r.users)\n  r = client.users.list(cursor=r.cursor) # get a response of users using cursor\n```\n\nIf you have installed it with the extras \'cli\', you can use the command line below in your shell.\n\n```sh\n$ kakaowork --help\nUsage: kakaowork [OPTIONS] COMMAND [ARGS]...\n\nOptions:\n  -k, --app-key TEXT\n  --help              Show this message and exit.\n\nCommands:\n  bots\n  conversations\n  departments\n  messages\n  spaces\n  users\n\n$ kakaowork -k <your_app_key> bots info\nID:     1\nName:   Test\nStatus: activated\n```\n\n## Contributing\nPull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.\n\nPlease make sure to update tests as appropriate.\n\n## [License](LICENSE)\n\nCopyright (c) 2021 Sung-Kyu Yoo.\n\nThis project is MIT license.\n',
    'author': 'Sung-Kyu Yoo',
    'author_email': 'skyoo2003@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/skyoo2003/kakaowork-py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
